#ifndef GAMESTATE_H
#define GAMESTATE_H

#include <types.h>
#include "GameTypes.h"
#include "Difficulty.h"

// The number of non player friendly ships on the map.
extern u8 friendlyShipCount;

// Indicates if the game cursor is current visible.
extern u8 displayCursor;

// The money for the whole game session.
extern u16 gameMoney;

// Indicates if the game is currently running within the game loop.
extern u8 gameRunning;

// Determines if the player units, friendly ships or mines are currently having their turn.
extern u8 gameTurnState;

// Determines if the hud should be redrawn on the next iteration of the game loop.
extern u8 hudRedraw;

// The unit index of the ship that is escorting.
extern u8 escortShipIndex;

// Indicates if the level has ended and whether it was with success or failure.
extern u8 levelEnded;

// The currently chosen level in the game.
extern u8 currentLevel;

// Determines when a map redraw should happen
extern u8 mapRedraw;

// The number of mines in the level.
extern u8 mineCount;

// The number of turns that the player has remaining.
extern u16 remainingTurns;

// The unit slot currently selected by the player.
extern u8 selectedUnitSlot;

// The unit that is currently active and so moved by arrows instead of using the cursor.
extern u8 activeUnit;

// Indicates if the units should be redrawn or not.
extern u8 unitRedraw;

// Indicates if all levels are accessible.
extern u8 unlockLevels;

// Resets the state of the game at the start of the game session.
void resetGameSessionState();

#endif