#ifndef GAMETYPES_H
#define GAMETYPES_H

#include <types.h>

// Screen dimensions.
#define SCREEN_X 80
#define SCREEN_Y 200

// Number of tiles that make up the screen.
#define SCREEN_TILES_Y 11
#define SCREEN_TILES_X 20

// Number of tiles that make up the map.
#define MAP_TILES_Y 36
#define MAP_TILES_X 60

// Full screen dimensions of the map.
#define MAP_SCREEN_Y 576
#define MAP_SCREEN_X 240

// Number of cells in the map (width * height).
#define MAP_CELL_COUNT 2160

// Size of tiles.
#define TILE_HEIGHT 16
#define TILE_WIDTH_BYTES 4

// Cell type flags.
#define CELL_EMPTY 0
#define CELL_ITEM_FLAG 0x10
#define CELL_INTERACTABLE_FLAG 0x20
#define CELL_MINE_EXPLOSION_FLAG 0x80

// The possible bits that the cell graphic could be in.
#define CELL_GFX_RANGE 0x0F

// Boolean values.
#define TRUE 1
#define FALSE 0

// Positions (x,y) of cells in the grid.
extern const u8 xPositions[20];
extern const u8 yPositions[12];

// Distance around a cell position.
// Start at West (W, NW, N, NE, E, SE, S, SW, W).
extern const i8 xAroundCellPositions[9];
extern const i8 yAroundCellPositions[9];

// Level and game states
#define LEVEL_READY 0
#define LEVEL_END_COMPLETE 1
#define LEVEL_END_DEATH 2
#define LEVEL_LOCKED 3
#define GAME_COMPLETE 4
#define GAME_OVER 5

// End address of video memory.
#define VIDEO_MEMORY_END (void*)(0xFFFF)

// Directions of movement.
#define DIRECTION_LEFT 1
#define DIRECTION_UP 2
#define DIRECTION_RIGHT 3
#define DIRECTION_DOWN 4

// The number of directions of movement.
#define DIRECTION_COUNT 4

// The amount the mine or friendly ship will move in one turn.
extern const i8 xDirectionChange[DIRECTION_COUNT];
extern const i8 yDirectionChange[DIRECTION_COUNT];

// The maximum number of units that the player can have.
#define MAX_PLAYER_UNIT_COUNT 8

// The maximum number of non player friendly ships the map can have.
#define MAX_FRIENDLY_SHIP_COUNT 4

// The maximum number of units (player, and non player) in the game.
#define MAX_UNIT_COUNT 26

// The position in the unit array that the non player friendly ships start.
#define FRIENDLY_SHIP_UNIT_START 8

// Game turn states.
// The player's unit turns.
#define PLAYER_TURN 0

// The non player friendly ships turn.
#define FRIENDLY_SHIP_TURN 8

// The floating mines turn.
#define MINE_TURN 12

#define PRE_PLAYER_TURN 32

#ifndef WINCPCTELERA
#define NULL 0
#endif

// The number of mine elements in the mine array.
// TODO increase value if I need more than 100 mines on a map.
#define MAX_MINE_ELEMENT_COUNT 300

// The first mine explosion graphic that the mine will change to on detonation.
#define MINE_EXPLOSION_GRAPHIC_START 5
#define MINE_EXPLOSION_GRAPHIC_MID 7
#define MINE_EXPLOSION_GRAPHIC_END 9

// The graphic shown when a bottom mine is revealed.
#define MINE_TYPE_BOTTOM_REVEALED_GRAPHIC 10

// The graphic shown when explosives are set onto the mine.
#define MINE_EXPLOSIVE_SET_GRAPHIC_START 11

// The last mine explosive set graphic before it will be changed into an explosion.
#define MINE_EXPLOSIVE_SET_GRAPHIC_END 14

// The types of mines in the game.
#define MINE_TYPE_MOOR_0 0
#define MINE_TYPE_MOOR_1 1
#define MINE_TYPE_FLOAT_0 2
#define MINE_TYPE_FLOAT_1 3
#define MINE_TYPE_BOTTOM_0 4

#define MINE_TYPE_COUNT 5

#endif