#include "Hud.h"

u8 hudRedraw;

#define HUD_X_POSITION_ 0
#define HUD_X_POSITION_TWO_ 64
#define HUD_Y_POSITION_ 176
#define HUD_WIDTH_ONE_ 64
#define HUD_WIDTH_TWO_ 16
#define HUD_HEIGHT_ 16

// X position of the hud action sprite.
#define HUD_X_UNIT_ACTION_ 4

// Half height is used for the indicators around the units.
#define TILE_HALF_HEIGHT_ 8

// Positions for the units on the hud.
#define HUD_Y_UNIT_POSITION_ 184
#define HUD_X_UNIT_START_ 10
#define HUD_X_UNIT_SEPARATION_ 8

#define HUD_ITEM_WIDTH_BYTES_ 3
#define HUD_ITEM_HUD_HEIGHT_ 8

#define HUD_TURN_START_ 0
#define REMAINING_TURN_START_ 11
#define HUD_MINE_START_ 18
#define MINE_COUNT_START_ 19
#define HUD_MONEY_START_ 27

#define HUD_CHANCE_START 41

void drawHud()
{
    u8 unitHudColour = greenMode ? 4 : 2;

    if (hudRedraw)
    {
        static u8 hudText[6];
        u8 i = 0;

        // Draw the hud strip on the background in two parts.
        u8* memory = cpct_getScreenPtr(CPCT_VMEM_START, HUD_X_POSITION_, HUD_Y_POSITION_ + 8);
        cpct_drawSolidBox(memory, cpct_px2byteM0(unitHudColour, unitHudColour), HUD_WIDTH_ONE_, HUD_HEIGHT_);
        cpct_drawSolidBox(memory + HUD_X_POSITION_TWO_, cpct_px2byteM0(unitHudColour, unitHudColour), HUD_WIDTH_TWO_, HUD_HEIGHT_);

        // Remaining turn count.
        drawString(HUD_TURN_START_, HUD_Y_POSITION_, "turns");
        convertNumberToString(remainingTurns, hudText);
        drawString(REMAINING_TURN_START_, HUD_Y_POSITION_, hudText);

        // Mine count.
        memory = cpct_getScreenPtr(CPCT_VMEM_START, HUD_MINE_START_, HUD_Y_POSITION_);
        cpct_drawSprite(G_hud_mine_0, memory, HUD_ITEM_WIDTH_BYTES_, HUD_ITEM_HUD_HEIGHT_);
        convertNumberToString(mineCount, hudText);
        drawString(MINE_COUNT_START_ + HUD_ITEM_WIDTH_BYTES_, HUD_Y_POSITION_, hudText);

        // Chances count.        
        drawString(HUD_CHANCE_START, HUD_Y_POSITION_, "used chances");
        drawString(HUD_CHANCE_START + 29, HUD_Y_POSITION_, "of");

        if (currentLevel == LEVEL_WILDLIFE)
        {
            convertNumberToString(turtlesKilled, hudText);
            drawString(HUD_CHANCE_START + 25, HUD_Y_POSITION_, hudText);
            convertNumberToString(allowedKilledTurtles, hudText);
            drawString(HUD_CHANCE_START + 34, HUD_Y_POSITION_, hudText);
        }
        else if (currentLevel == LEVEL_HISTORICAL_STRUCTURES)
        {
            convertNumberToString(destroyedStructures, hudText);
            drawString(HUD_CHANCE_START + 25, HUD_Y_POSITION_, hudText);
            convertNumberToString(allowedDestroyedStructures, hudText);
            drawString(HUD_CHANCE_START + 34, HUD_Y_POSITION_, hudText);
        }
        else
        {
            convertNumberToString(currentLeveShipArraySize / 5 - friendlyShipCount, hudText);
            drawString(HUD_CHANCE_START + 25, HUD_Y_POSITION_, hudText);
            convertNumberToString(allowedDestroyedShips, hudText);
            drawString(HUD_CHANCE_START + 34, HUD_Y_POSITION_, hudText);
        }

        // Player money.
        memory = cpct_getScreenPtr(CPCT_VMEM_START, HUD_MONEY_START_, HUD_Y_POSITION_);
        cpct_drawSprite(G_hud_money_0, memory, HUD_ITEM_WIDTH_BYTES_, HUD_ITEM_HUD_HEIGHT_);
        convertNumberToString(gameMoney, hudText);

        // +1 is for a gap which currently looks better but probably will be removed when hud colour is changed.
        drawString(HUD_MONEY_START_ + HUD_ITEM_WIDTH_BYTES_ + 1, HUD_Y_POSITION_, hudText);

        // Unit action graphics.
        memory = cpct_getScreenPtr(CPCT_VMEM_START, HUD_X_UNIT_ACTION_, HUD_Y_UNIT_POSITION_);
        
        if (units[selectedUnitSlot].deployed)
        {
            if (units[selectedUnitSlot].defusing)
            {
                if (units[selectedUnitSlot].type == UNIT_TYPE_RADAR)
                {
                    // Radar ship unit is doing a scan (or cooling down from a scan).
                    cpct_drawSprite(G_hud_action_radar, memory, TILE_WIDTH_BYTES, TILE_HEIGHT);
                }
                else if (units[selectedUnitSlot].type == UNIT_TYPE_EXPLOSIVE_DIVER)
                {
                    // Unit is setting a charge.
                    cpct_drawSprite(G_hud_action_explosive, memory, TILE_WIDTH_BYTES, TILE_HEIGHT);
                }
                else
                {
                    // Unit is defusing the mine.
                    cpct_drawSprite(G_hud_action_defusing, memory, TILE_WIDTH_BYTES, TILE_HEIGHT);
                }
            }
            else if (units[selectedUnitSlot].target.x != 255 || units[selectedUnitSlot].target.y != 255)
            {
                cpct_drawSprite(G_hud_action_arrow, memory, TILE_WIDTH_BYTES, TILE_HEIGHT);
            }
            else
            {
                if (playerUnitTurnFlag & (1 << selectedUnitSlot))
                {
                    // Unit is free to move action.
                    cpct_drawSprite(G_hud_action_move, memory, TILE_WIDTH_BYTES, TILE_HEIGHT);
                }
                else
                {
                    // Unit has finished turn and cannot move now.
                    cpct_drawSprite(G_hud_action_move_red, memory, TILE_WIDTH_BYTES, TILE_HEIGHT);
                }
            }
        }
        else
        {
            // Clear the action if the unit has not been deployed to the map.
            cpct_drawSolidBox(memory, cpct_px2byteM0(unitHudColour, unitHudColour), TILE_WIDTH_BYTES, TILE_HEIGHT);
        }

        // Draws the slots for the player's current units.
        memory = cpct_getScreenPtr(CPCT_VMEM_START, HUD_X_UNIT_START_, HUD_Y_UNIT_POSITION_);

        // Displays player's currently chosen units.
        for (i = 0; i < MAX_PLAYER_UNIT_COUNT; ++i)
        {
            // Draw a box around the units that have been deployed to the map.
            if (units[i].deployed)
            {
                if ((playerUnitTurnFlag & (1 << i)))
                {
                    // Unit is ready so display green indicator.
                    // +79 gets us to the bottom half of the unit.
                    cpct_drawSolidBox(memory + 79, cpct_px2byteM0(12, 12), TILE_WIDTH_BYTES + 2, TILE_HALF_HEIGHT_);
                }
                else
                {
                    // Unit has completed their turn so display red indicator.
                    // +79 gets us to the bottom half of the unit.
                    cpct_drawSolidBox(memory + 79, cpct_px2byteM0(3, 3), TILE_WIDTH_BYTES + 2, TILE_HALF_HEIGHT_);
                }
            }

            // Draw a box around the top half of the currently selected unit slot (either side of the box).
            if (selectedUnitSlot == i)
            {
                cpct_drawSolidBox(memory - 1, cpct_px2byteM0(5, 5), TILE_WIDTH_BYTES + 2, TILE_HALF_HEIGHT_);
            }

            // When no unit is assigned to the unit number a solid box will be displayed.
            if (units[i].type == 255)
            {
                cpct_drawSolidBox(memory, cpct_px2byteM0(6, 6), TILE_WIDTH_BYTES, TILE_HEIGHT);
            }
            else
            {
                cpct_drawSprite(hudUnitGraphics[units[i].type], memory, TILE_WIDTH_BYTES, TILE_HEIGHT);

                // Draw an x if the unit is dead.
                if (units[i].dead)
                {
                    cpct_drawSpriteMasked(G_unit_destroyed_x, memory, TILE_WIDTH_BYTES, TILE_HEIGHT);
                }
            }

            memory += HUD_X_UNIT_SEPARATION_;
        }

        hudRedraw = FALSE;
    }
}

void initHud()
{
    hudRedraw = TRUE;
}