#ifndef KEYS_H
#define KEYS_H

#include <cpctelera.h>

// First set of controls.
#define CONTROL_UP_FLAG 1
#define CONTROL_DOWN_FLAG 2
#define CONTROL_LEFT_FLAG 4
#define CONTROL_RIGHT_FLAG 8
#define CONTROL_USE_FLAG 16
#define CONTROL_EXIT_FLAG 32
#define CONTROL_TARGET_FLAG 64
#define CONTROL_DEFUSE_FLAG 128

// Second set of controls.
#define CONTROL_END_TURN_FLAG 1

// Flags for the controls to select each unit number\group.
#define CONTROL_UNIT_NUMBER_FLAG_1 1
#define CONTROL_UNIT_NUMBER_FLAG_2 2
#define CONTROL_UNIT_NUMBER_FLAG_3 4
#define CONTROL_UNIT_NUMBER_FLAG_4 8
#define CONTROL_UNIT_NUMBER_FLAG_5 16
#define CONTROL_UNIT_NUMBER_FLAG_6 32
#define CONTROL_UNIT_NUMBER_FLAG_7 64
#define CONTROL_UNIT_NUMBER_FLAG_8 128

typedef struct Keys Keys;

// Defines the controls used in the game.
struct Keys
{
    // Key values for setting custom controls.
    cpct_keyID up;
    cpct_keyID down;
    cpct_keyID left;
    cpct_keyID right;
    cpct_keyID use;
    cpct_keyID exit;
    cpct_keyID target;
    cpct_keyID defuse;
    cpct_keyID endTurn;
    
    // Storing control flags in two u8 uses less memory than a single u16.
    u8 controlFlags;
    u8 controlFlags2;
    
    // Unit number hokey flags.
    u8 unitNumberFlags;
};

// Initialises key values so that no keys are pressed at the start of the game.
void initKeys();

// Updated the controls flags based on the keys that are currently down.
void readGameControls();

// Sets the default player controls for the game.
void setDefaultKeys();

extern Keys gameControls;

#endif