#include "Map.h"
#include <stdio.h>

u8 mapCells[MAP_CELL_COUNT];

u8 mapMines[MAX_MINE_ELEMENT_COUNT];

u8* currentLevelShips;
u8 currentLeveShipArraySize;
u8* currentLevelTargets;

void drawMap()
{
    if (mapRedraw)
    {
        // Draw the content of the mapArray (map grid).
        u8 x = 0;
        u8 y = 0;
        u8 cell = 0;
        u16 i = 0;

        u16 position = XY_TO_POSITION(xScroll, yScroll);
        u8* cellPointer = mapCells + position;

        // Position is the element in the map tile array.
        for (x = 0; x < SCREEN_TILES_X; ++x)
        {
            for (y = 0; y < SCREEN_TILES_Y; ++y)
            {
                cell = *cellPointer;

                drawCell(x, y, cell);

                ++cellPointer;
            }

            // Move to the start of the next column (half the column height in a 2x2 screen map), 2/3 of the column height in a 3x3 map.
            // Changed from 24 to 25 due to changing screen height to 11.
            cellPointer += 25;
        }

        for (i = 0; i < level_mines_array_size[currentLevel]; i += 3)
        {
            u8 mineX = mapMines[i];
            u8 mineY = mapMines[i + 1];

            if (mineX >= xScroll && mineX < xScroll + SCREEN_TILES_X)
            {
                if (mineY >= yScroll && mineY < yScroll + SCREEN_TILES_Y)
                {
                    u8 x = xPositions[mineX - xScroll];
                    u8 y = yPositions[mineY - yScroll];
                    u8* mapMemory = cpct_getScreenPtr(CPCT_VMEM_START, x, y);

                    drawMine(mapMines[i + 2], mapMemory);
                }
            }
        }

        mapRedraw = FALSE;
    }
}

void drawBlankCellScroll(u8 x, u8 y)
{
    u8* mapMemory = cpct_getScreenPtr(CPCT_VMEM_START, x - xScroll, y - yScroll);

    // Draw a blank box on screen.
    cpct_drawSolidBox(mapMemory, cpct_px2byteM0(0, 0), TILE_WIDTH_BYTES, TILE_HEIGHT);
}

void drawCell(u8 x, u8 y, u8 cell)
{
    u8* mapMemory = cpct_getScreenPtr(CPCT_VMEM_START, xPositions[x], yPositions[y]);

    if (cell & CELL_ITEM_FLAG)
    {
        // Draw item (-1 because the graphicset will start at 0 index but the type will start at 1).
        cpct_drawSprite(itemGraphics[(cell & CELL_GFX_RANGE) - 1], mapMemory, TILE_WIDTH_BYTES, TILE_HEIGHT);
    }
    else if (cell & CELL_INTERACTABLE_FLAG)
    {
        // Draw item (-1 because the graphicset will start at 0 index but the type will start at 1).
        cpct_drawSprite(interactableGraphics[(cell & CELL_GFX_RANGE) - 1], mapMemory, TILE_WIDTH_BYTES, TILE_HEIGHT);
    }
    else if (cell & CELL_MINE_EXPLOSION_FLAG)
    {
        // Draw blank cells as yellow for mine explosion blast radius.
        cpct_drawSolidBox(mapMemory, cpct_px2byteM0(1, 1), TILE_WIDTH_BYTES, TILE_HEIGHT);
    }
    else if (cell & CELL_GFX_RANGE)
    {
        // Draw tile (-1 because the graphicset will start at 0 index but the type will start at 1).
        cpct_drawSprite(tileGraphics[(cell & CELL_GFX_RANGE) - 1], mapMemory, TILE_WIDTH_BYTES, TILE_HEIGHT);
    }
    else
    {
        // Draw blank cells as blue for water.
        cpct_drawSolidBox(mapMemory, cpct_px2byteM0(0, 0), TILE_WIDTH_BYTES, TILE_HEIGHT);
    }
}

void drawMine(u8 mine, u8* memory)
{
    // Don't draw bottom mine or invalid (255) mines on screen.
    if (mine != MINE_TYPE_BOTTOM_0 && mine != 255)
    {
        cpct_drawSprite(mineGraphics[mine], memory, TILE_WIDTH_BYTES, TILE_HEIGHT);
    }
}

u16 findMine(u8 x, u8 y)
{
    u16 i = 0;

    for (i = 0; i < level_mines_array_size[currentLevel]; i += 3)
    {
        // Look for a mine that is at (x,y) and has a valid type (not 255).
        if (mapMines[i] == x && mapMines[i + 1] == y && mapMines[i + 2] != 255)
        {
            // Returns the index of the mine type so we can work out the impact of a mine at this position by quering the array.
            // It can also be used to change the mine graphic to an exploded mine.
            return i + 2;
        }
    }

    // No mine was found at (x,y).
    return 0;
}

void initMap(u8 mapNumber)
{
    // The end of the source level array (arraypointer + size - 1) as a u16 pointer.
    void* sourceEnd = level_map[mapNumber] + level_map_size[mapNumber] - 1;

    // The end of the destination array (arraypointer + size - 1) as a u16 pointer.
    void* destinationEnd = mapCells + MAP_CELL_COUNT - 1;

    // Enable map redraw.
    mapRedraw = TRUE;

    mineCount = level_mine_count[mapNumber];

    // Sets mine for the map.
    if (mineCount > 0)
    {
        cpct_memcpy(mapMines, level_mines[currentLevel], level_mines_array_size[currentLevel]);
    }

    // Loading map in crunched form.
    cpct_zx7b_decrunch_s(destinationEnd, sourceEnd);

    // Setting current level ship variables.
    currentLevelShips = level_ship[mapNumber];
    currentLeveShipArraySize = level_ship_array_size[mapNumber];

    // Setting current level target pointer.
    currentLevelTargets = level_target[mapNumber];

    // The number of turns that the player has remaining for the level.
    remainingTurns = level_turns[mapNumber];
}