#include "MapScreen.h"

#define TITLE_X_ 20
#define TITLE_Y_ 10
#define LEVEL_X_ 20
#define LEVEL_Y_0_ 30

void drawLevelStateIndicators(u8* memory, u8 levelIndex)
{
    u8 levelState = levelStates[levelIndex];

    if (levelState == LEVEL_END_COMPLETE)
    {
        cpct_drawSprite(G_level_state_tick, memory, TILE_WIDTH_BYTES, TILE_HEIGHT);
    }
    else if (levelState == LEVEL_LOCKED)
    {
        cpct_drawSprite(G_level_state_lock, memory, TILE_WIDTH_BYTES, TILE_HEIGHT);
    }
}

void drawMapScreen()
{
    u8 i = 0;

    cpct_clearScreen(0);

    drawString(TITLE_X_, TITLE_Y_, "select a level");

    // Draw a list of levels.
    for (i = 0; i < LEVEL_COUNT; ++i)
    {
        u8 y = LEVEL_Y_0_ + i * 18;
        u8* memory = cpct_getScreenPtr(CPCT_VMEM_START, LEVEL_X_ - 7, y - 4);

        // Level number (add 1 to get user friendly number).
        drawCharacter(LEVEL_X_ - 6, y, i + 1);
        drawString(LEVEL_X_ - 4, y, ".");

        // The indicator will cover up the number.
        drawLevelStateIndicators(memory, i);
        
        // Level name.
        drawString(LEVEL_X_, y, levelNames[i]);
    }
}

void runMapScreen()
{
    u8 choosingMap = TRUE;
    u8 i = 0;
    u8* memory;

    drawMapScreen();

    currentLevel = 255;

    // Wait for any previous key presses to clear first.
    while (cpct_isAnyKeyPressed());

#ifdef WINCPCTELERA
    // This is to make it harder to accidently go from main menu 1 to the first level.
    while (!cpct_isAnyKeyPressed());
#endif

    while (choosingMap)
    {
        // Read the controls the user is pressing.
        readGameControls();

        // 1 to 8 to choose the level.
        for (i = 0; i < 8; ++i)
        {
            // Choose the unit number flags 1 to 8.
            if (gameControls.unitNumberFlags & 1 << i)
            {
                // Level gating.
                if (unlockLevels || levelStates[i] == LEVEL_READY)
                {
                    currentLevel = i;
                }
            }
        }

        // Move to next screen on choosing a level.
        if (currentLevel != 255)
        {
            // Updated level ended so the game is ready for the next level.
            levelEnded = LEVEL_READY;

            choosingMap = FALSE;
        }

        // Check that the exit key has been pressed.
        if (gameControls.controlFlags & CONTROL_EXIT_FLAG)
        {
            // If the player choses to press yes exit the game.
            if (exitDialog())
            {
                choosingMap = FALSE;
                levelEnded = GAME_OVER;
            }
            else
            {
                drawMapScreen();
            }
        }
    }
}