#include "Menu.h"

#define GAME_VERSION_ "1.0"

// Colour mode text.
const u8* COLOUR_MODE_MENU_TEXT_;
const u8 GREEN_MODE_TEXT_[14] = "5. green mode";
const u8 COLOUR_MODE_TEXT_[15] = "5. colour mode";

// Difficulty level text.
const u8* difficultyMenuText_;
const u8 EASY_DIFFICULTY_TEXT_[19] = "3. easy difficulty";
const u8 NORMAL_DIFFICULTY_TEXT_[21] = "3. normal difficulty";
const u8 HARD_DIFFICULTY_TEXT_[19] = "3. hard difficulty";

#define SCORE_COUNT_ 5
#define MAX_NAME_LENGTH_ 10

u16 highScores_[SCORE_COUNT_];
u8 highScoreNames_[SCORE_COUNT_][11];

u8 unlockLevels = FALSE;

#define START_GAME_KEY_ Key_1
#define REBIND_CONTROLS_KEY_ Key_2
#define DIFFICULTY_KEY_ Key_3
#define HIGH_SCORES_KEY_ Key_4
#define COLOUR_MODE_KEY_ Key_5

void chooseDifficulty()
{
    // Switch between difficulties and upate the menu text.
    if (chosenDifficulty == EASY_DIFFICULTY)
    {
        chosenDifficulty = NORMAL_DIFFICULTY;
        difficultyMenuText_ = NORMAL_DIFFICULTY_TEXT_;
    }
    else if (chosenDifficulty == NORMAL_DIFFICULTY)
    {
        chosenDifficulty = HARD_DIFFICULTY;
        difficultyMenuText_ = HARD_DIFFICULTY_TEXT_;
    }
    else
    {
        chosenDifficulty = EASY_DIFFICULTY;
        difficultyMenuText_ = EASY_DIFFICULTY_TEXT_;
    }
}

void displayEndScreen()
{
    // Wait for any previous key presses to clear first.
    while (cpct_isAnyKeyPressed());

    // Clear the screen
    cpct_clearScreen(0);

    if (levelEnded == GAME_COMPLETE)
    {
        u8* memory;
        u8 count = 255;
        u8 colour = 0;
        u8 x = 0;
        u8 y = 0;

        // Draw a patterned background.
        while (count)
        {
            colour = 1 + cpct_rand() % 15;
            y = 14 * (cpct_rand() % 14);
            x = 4 * (cpct_rand() % 17);

            memory = cpct_getScreenPtr(CPCT_VMEM_START, x, y);
            cpct_drawSolidBox(memory, cpct_px2byteM0(colour, colour), 10, 12);

            --count;
        }

        // Draw victorious text on screen.
        memory = cpct_getScreenPtr(CPCT_VMEM_START, 25, 60);
        cpct_drawSolidBox(memory, cpct_px2byteM0(0, 0), 30, 20);
        drawString(30, 65, "victorious");

        // Add to money on winning the game as this is used in place of score.
        gameMoney += 1000;
    }
    else
    {
        // Display Game Over.
        drawString(30, 20, "game over");
    }

    //then wait for the desired key press
    while (!cpct_isAnyKeyPressed());
}

void displayHighScores()
{
    u8 i = 0;    

    cpct_clearScreen(0);

    // High scores title.
    drawString(25, 5, "high scores");

    // Each of the entires in the high score list.
    for (i = 0; i < SCORE_COUNT_; ++i)
    {
        u8 scoreStart = 25;
        static u8 output[6];
        u8 y = 45 + i * 15;

        // Draw the name of the player from the names array.
        drawString(40, y, highScoreNames_[i]);
        
        // Convert numeric score to a string for on the scores list.
        convertNumberToString(highScores_[i], output);
        
        // Draw all zeros.
        drawString(scoreStart, y, "00000");

        // Determines where the score is drawn so there are 0s in the right place.
        if (highScores_[i] < 100)
        {
            scoreStart += 6;
        }
        else if (highScores_[i] < 1000)
        {
            scoreStart += 4;
        }
        else if (highScores_[i] < 10000)
        {
            scoreStart += 2;
        }

        drawString(scoreStart, y, output);
    }

    getKey();
}

void drawMenu()
{
    // Clear the screen before drawing the menu.
    cpct_clearScreen(0);

    // Game title.
    drawString(18, 5, "fitzroy goes sweeping");

    // Menu item choices.
    drawString(20, 45, "1. start game");
    drawString(20, 60, "2. rebind controls");
    drawString(20, 75, difficultyMenuText_);
    drawString(20, 90, "4. high scores");
    drawString(20, 105, COLOUR_MODE_MENU_TEXT_);

    // Draw the game version number on the bottom right corner of the screen.
    drawString(70, 180, GAME_VERSION_);
}

cpct_keyID getKey()
{
    // From Ronaldo and Arnaud on cpcwiki forum.
    u8* keyStatus = cpct_keyboardStatusBuffer;
    u8 i;

    // Wait for any previous key presses to clear first.
    while (cpct_isAnyKeyPressed());

    // Then wait for the desired key press.
    while (!cpct_isAnyKeyPressed());

    for (i = 0; i < 10; ++i)
    {
        cpct_keyID keypressed = *keyStatus++ ^ 0xFF;
        if (keypressed)
        {
            return (keypressed << 8) + i;
        }
    }

    return 0;
}

void initHighScores()
{
    u8 i = 0;

    for (i = 0; i < SCORE_COUNT_; ++i)
    {
        // TODO update to the starting money value once decided.
        highScores_[i] = 100;
        cpct_memcpy(highScoreNames_[i], "cpc\0", 4);
    }
}

void rebindControls()
{
    // Clear the screen so that we can show control text.
    cpct_clearScreen(0);

    // Enter controls title.
    drawString(15,8, "enter controls");

    // Draw the text indicating which control is being set.
    // Make sure no key is currently down.
    // Set the key for the next control (this is all for each getKey call).
    drawString(25, 30, "up");
    gameControls.up = getKey();

    drawString(25, 45, "down");
    gameControls.down = getKey();

    drawString(25, 60, "left");
    gameControls.left = getKey();

    drawString(25, 75, "right");
    gameControls.right = getKey();

    drawString(25, 90, "deploy or unit mode change");
    gameControls.use = getKey();

    drawString(25, 105, "action or defuse");
    gameControls.defuse = getKey();

    drawString(25, 120, "set target");
    gameControls.target = getKey();

    drawString(25, 135, "end turn");
    gameControls.endTurn = getKey();

    drawString(25, 150, "exit");
    gameControls.exit = getKey();
}

void runMenu()
{
    unlockLevels = FALSE;

    // Set the default high score entries.
    initHighScores();

    // Set the default keys for the player.
    setDefaultKeys();

    // Default graphics mode is colour mode.
    COLOUR_MODE_MENU_TEXT_ = COLOUR_MODE_TEXT_;

    // Default difficulty is normal difficulty.
    chosenDifficulty = NORMAL_DIFFICULTY;
    difficultyMenuText_ = NORMAL_DIFFICULTY_TEXT_;

    // Draw the menu initially on entry to the main menu.
    drawMenu();

    // Main menu loop.
    while (1)
    {
        if (cpct_isKeyPressed(START_GAME_KEY_))
        {
            // Runs the full game session.
            runGameSession();

            // Change to level complete for testing victory screen.
            //levelEnded = LEVEL_END_COMPLETE;

            // Displays either game over or victory screen depending on if the player suceeded or not.
            displayEndScreen();

            // Update and show high scores.
            updateHighScores();

            // Redraw the menu after finishing the game session.
            drawMenu();
        }
        else if (cpct_isKeyPressed(REBIND_CONTROLS_KEY_))
        {
            rebindControls();

            // Redraw the menu after leaving rebind controls menu.
            drawMenu();
        }
        else if (cpct_isKeyPressed(DIFFICULTY_KEY_))
        {
            chooseDifficulty();
            drawMenu();
        }
        else if (cpct_isKeyPressed(HIGH_SCORES_KEY_))
        {
            displayHighScores();

            // Redraw the menu after leaving the high scores.
            drawMenu();
        }
        else if (cpct_isKeyPressed(COLOUR_MODE_KEY_))
        {
            switchColourMode();

            // Redraw the menu after updating the menu item text.
            drawMenu();
        }

        if (greenMode && cpct_isKeyPressed(Key_G))
        {
            unlockLevels = TRUE;
        }
    }
}

void switchColourMode()
{
    // Update menu item text to reflect the chosen colour mode.
    if (greenMode)
    {
        greenMode = FALSE;
        COLOUR_MODE_MENU_TEXT_ = COLOUR_MODE_TEXT_;
    }
    else
    {
        greenMode = TRUE;
        COLOUR_MODE_MENU_TEXT_ = GREEN_MODE_TEXT_;
    }
}

void updateHighScores()
{
    // Check that the player got a higher score (money) than the bottom score.
    if (gameMoney > highScores_[4])
    {
        // Let the player enter their name.
        u8 letterCount = 0;
        static u8 scoreName[11];
        static u8 highScoreText[6];
        u8 i = 0;

        // Default the chosen key to whatever.
        cpct_keyID chosenKey = Key_0;

        // Set scoreName to the null terminator to end the string.
        cpct_memset(scoreName, '\0', 11);

        cpct_clearScreen(0);
        drawString(15, 5, "new high score");        
        convertNumberToString(gameMoney, highScoreText);
        drawString(45, 5, highScoreText);
        drawString(20, 25, "enter your name");

        do
        {
            // Get the next character from the user.
            chosenKey = getKey();

            // Del to move back to the previous key.
            if (chosenKey == Key_Del)
            {
                // Go back to the previous letter.
                if (letterCount)
                {
                    --letterCount;

                    scoreName[letterCount] = ' ';

                    // Draw letter showing a space where it used to be a letter.
                    drawString(25 + letterCount * 2, 50, &scoreName[letterCount]);
                }
            }

            // Make sure the key is in the correct range and the word is not too long.
            else if (letterCount < MAX_NAME_LENGTH_)
            {
                u8 chosenCharacter = 0;

                switch (chosenKey)
                {
                case Key_A:
                    chosenCharacter = 'a';
                    break;

                case Key_B:
                    chosenCharacter = 'b';
                    break;

                case Key_C:
                    chosenCharacter = 'c';
                    break;

                case Key_D:
                    chosenCharacter = 'd';
                    break;

                case Key_E:
                    chosenCharacter = 'e';
                    break;

                case Key_F:
                    chosenCharacter = 'f';
                    break;

                case Key_G:
                    chosenCharacter = 'g';
                    break;

                case Key_H:
                    chosenCharacter = 'h';
                    break;

                case Key_I:
                    chosenCharacter = 'i';
                    break;

                case Key_J:
                    chosenCharacter = 'j';
                    break;

                case Key_K:
                    chosenCharacter = 'k';
                    break;

                case Key_L:
                    chosenCharacter = 'l';
                    break;

                case Key_M:
                    chosenCharacter = 'm';
                    break;

                case Key_N:
                    chosenCharacter = 'n';
                    break;

                case Key_O:
                    chosenCharacter = 'o';
                    break;

                case Key_P:
                    chosenCharacter = 'p';
                    break;

                case Key_Q:
                    chosenCharacter = 'q';
                    break;

                case Key_R:
                    chosenCharacter = 'r';
                    break;

                case Key_S:
                    chosenCharacter = 's';
                    break;

                case Key_T:
                    chosenCharacter = 't';
                    break;

                case Key_U:
                    chosenCharacter = 'u';
                    break;

                case Key_V:
                    chosenCharacter = 'v';
                    break;

                case Key_W:
                    chosenCharacter = 'w';
                    break;

                case Key_X:
                    chosenCharacter = 'x';
                    break;

                case Key_Y:
                    chosenCharacter = 'y';
                    break;

                case Key_Z:
                    chosenCharacter = 'z';
                    break;

                case Key_Space:
                    chosenCharacter = ' ';
                    break;
                }

                if (chosenCharacter)
                {
                    scoreName[letterCount] = chosenCharacter;

                    // Draw letter, the one just selected.
                    drawString(25 + letterCount * 2, 50, &scoreName[letterCount]);

                    ++letterCount;
                }
            }
        } while (chosenKey != Key_Return && chosenKey != Key_Enter);

        // Update score values.
        highScores_[4] = gameMoney;

        for (i = 4; i > 0; --i)
        {
            if (highScores_[i] > highScores_[i - 1])
            {
                highScores_[i] = highScores_[i - 1];
                cpct_memcpy(highScoreNames_[i], highScoreNames_[i - 1], 11);

                highScores_[i - 1] = gameMoney;
                cpct_memcpy(highScoreNames_[i - 1], scoreName, 11);
            }
        }
    }
}
