#include "Mine.h"
#include <stdio.h>
void logicMines()
{
    if (gameTurnState == MINE_TURN)
    {
        u16 i = 0;

        for (i = 0; i < level_mines_array_size[currentLevel]; i += 3)
        {
            u8 type = mapMines[i + 2];
            
            // Check that the mine is a floating mine.
            if (type == MINE_TYPE_FLOAT_0 || type == MINE_TYPE_FLOAT_1)
            {
                u8 j = 0;
                u8 x = mapMines[i];
                u8 y = mapMines[i + 1];

                // Determine possible directions so if one is blocked the mine can move a different way.
                determineRandomDirections();

                // Iterate through the random directions so that if it is blocked on one it will chose another.
                for (j = 0; j < DIRECTION_COUNT; ++j)
                {
                    u8 direction = determinedDirections[j];

                    // Determines the next position for the mine based on the determined random direction.
                    u8 nextX = x + xDirectionChange[direction];
                    u8 nextY = y + yDirectionChange[direction];

                    // Make sure next x,y is not outside the range of the map.
                    if (nextX >= MAP_TILES_X)
                    {
                        continue;
                    }
                    else if (nextY >= MAP_TILES_Y)
                    {
                        continue;
                    }
                    else if (processCellMine(nextX, nextY))
                    {
                        // The cell is blocked by something in the cell.
                        continue;
                    }

                    // Update the mine position to the newly determined (random) position.
                    mapMines[i] = nextX;
                    mapMines[i + 1] = nextY;

                    // Successfully updated the mine position so break out of the direction loop.
                    break;
                }
            }
            else if (type >= MINE_EXPLOSION_GRAPHIC_START && type <= MINE_EXPLOSION_GRAPHIC_END)
            {
                // Cycle through mine explosion graphics.
                if (mapMines[i + 2] < MINE_EXPLOSION_GRAPHIC_MID)
                {
                    ++mapMines[i + 2];

                    // Expanding the explosion.
                    updateCellExplosionFlags(mapMines[i], mapMines[i + 1], mapMines[i + 2] - MINE_EXPLOSION_GRAPHIC_START, TRUE);
                }
                else if (mapMines[i + 2] < MINE_EXPLOSION_GRAPHIC_END)
                {
                    // Contracting the explosion
                    // Comes in as MINE_EXPLOSION_GRAPHIC_END - 7 the first time to clear the outer ring.
                    // Then we increment mine type so it is MINE_EXPLOSION_GRAPHIC_END - 8 so we clear the inner ring.
                    updateCellExplosionFlags(mapMines[i], mapMines[i + 1], MINE_EXPLOSION_GRAPHIC_END - mapMines[i + 2], FALSE);

                    // This must be done after updating the flags.
                    ++mapMines[i + 2];
                }
                else
                {
                    // Explosion complete so set the mine type to 255 effectively removing it.
                    mapMines[i + 2] = 255;

                    // Update the mine count following the completion of the explosion.
                    --mineCount;

                    // Redraw the hud following updating the mine count.
                    hudRedraw = TRUE;
                }
            }
            else if (type >= MINE_EXPLOSIVE_SET_GRAPHIC_START && type <= MINE_EXPLOSIVE_SET_GRAPHIC_END)
            {
                // Cycle through mine mine explosive set graphics.
                if (mapMines[i + 2] < MINE_EXPLOSIVE_SET_GRAPHIC_END)
                {
                    ++mapMines[i + 2];
                }
                else
                {
                    // Explosive timer has expired so now the explosion can happen.
                    mapMines[i + 2] = MINE_EXPLOSION_GRAPHIC_START;
                }
            }
        }

        // Redraw the map after mine positions have changed.
        mapRedraw = TRUE;

        // Change to pre player turn after mine turns have completed.
        gameTurnState = PRE_PLAYER_TURN;
    }
}

u8 processCellMine(u8 x, u8 y)
{
    u8 cell = MAPA(x, y);
    u16 mineCell = findMine(x, y);

    if (cell & CELL_GFX_RANGE)
    {
        // If there is a graphic in the cell (it is an item, interactable or tile) then it is solid.
        // This means the mine cannot occupy the same space as another object.
        return 1;
    }
    else if (mineCell)
    {
        // Mine cell is returned as solid so two mines cannot collide with each other.
        // With random floating mines this would be problematic.
        return 1;
    }
    else
    {
        // Everything else would be an empty cell.
        return 0;
    }
}

void updateCellExplosionFlags(u8 x, u8 y, u8 radius, u8 expanding)
{
    // The mine at the centre of the explosion.
    u8 sourceMineX = x;
    u8 sourceMineY = y;

    // Determine the radius around the mine to update.
    u8 searchRadiusStartX = x - radius < 240 ? x - radius : 0;
    u8 searchRadiusStartY = y - radius < 240 ? y - radius : 0;
    u8 searchRadiusEndX = x + 1 + radius < MAP_TILES_X ? x + 1 + radius : MAP_TILES_X - 1;
    u8 searchRadiusEndY = y + 1 + radius < MAP_TILES_Y ? y + 1 + radius : MAP_TILES_Y - 1;

    for (x = searchRadiusStartX; x < searchRadiusEndX; ++x)
    {
        for (y = searchRadiusStartY; y < searchRadiusEndY; ++y)
        {
            u8 cell = MAPA(x, y);

            // Historical structures start non-zero so we can use 0 in the turnary statement.
            // All historical structures are at the end of the interactable graphics array.
            u8 interactable = cell & CELL_INTERACTABLE_FLAG ? (cell & 0x1F) >= INTERACTABLE_TYPE_HISTORICAL_STRUCTURE_0 : 0;

            // Check that cell is either empty, has an existing explosion set or is a historical structure.
            if (!cell || cell & CELL_MINE_EXPLOSION_FLAG || interactable)
            {
                // Check whether the explosion is expanding or not.
                if (expanding)
                {
                    u16 mine = findMine(x, y);
                    u8 unitIndex = containsUnit(x, y, NULL);

                    // Check if the cell already has an explosion going in it.
                    if (cell & CELL_MINE_EXPLOSION_FLAG)
                    {
                        // There is an existing explosion going on in the cell so we just blindly add to it.
                        MAPA(x, y) = cell + 1;
                    }
                    else
                    {
                        // There is no existing range so this is the first explosion on the cell including the flag.
                        MAPA(x, y) = CELL_MINE_EXPLOSION_FLAG + 1;
                    }

                    // Check that a mine has been found but that it isn't the source mine of the explosion.
                    if (mine && !(x == sourceMineX && y == sourceMineY))
                    {
                        // Make sure the mine has not been exploded.
                        if (mapMines[mine] < MINE_EXPLOSION_GRAPHIC_START || mapMines[mine] == MINE_TYPE_BOTTOM_REVEALED_GRAPHIC)
                        {
                            // Any mines that are found within the radius of the mine will be exploded.
                            mapMines[mine] = MINE_EXPLOSION_GRAPHIC_START;
                        }
                    }

                    if (unitIndex)
                    {
                        // -1 to get the correct index.
                        --unitIndex;

                        // Kill the unit at the location of the mine.
                        units[unitIndex].health = 0;

                        if (units[unitIndex].carryingCargo)
                        {
                            // Instant fail of objectives if the ship carrying the cargo gets destroyed.
                            failObjectives();
                        }

                        // If the unit that died was a kamakazi vescel then make a bigger boom!
                        deliverKamikaziPayload(&units[unitIndex]);

                        if (units[unitIndex].unitClass == SHIP_UNIT_CLASS)
                        {
                            // Update the friendly ship count on unit death.
                            --friendlyShipCount;
                        }
                        else if (units[unitIndex].unitClass == MARINE_LIFE_UNIT_CLASS)
                        {
                            // Lose money if the mine explosion kills the marine creature.
                            gameMoney -= MARINE_LIFE_MONEY_DEDUCTION;

                            // Update objectives, potentially ending the game.
                            ++turtlesKilled;
                        }
                        else
                        {
                            // When a unit is killed by a mine play an explosion.
                            playSound(SOUND_EXPLOSION);
                        }
                    }

                    // This is only true if it is a historical structure.
                    if (interactable)
                    {
                        // Explosion flag has already been set on the cell so we don't have to update the historical cell value                        
                        // TODO consider updating historical structure to rubble instead of blank.
                        
                        // Updates the objectives value for historical structures.
                        --destroyedStructures;
                    }
                }
                else
                {
                    // When we are removing the explosion radius check to see if there is only one explosion on the cell.
                    if ((cell & CELL_GFX_RANGE) == 1)
                    {
                        // When there is only a single explosion reset the value to 0 (removing the flag).
                        MAPA(x, y) = 0;
                    }
                    else
                    {
                        // There is more then on explosion on the cell so only subtract one.
                        MAPA(x, y) = cell - 1;
                    }
                }
            }
        }
    }
}