#include "Objectives.h"

u8 rescueCargo_ = 0;
u8 shipsToRescue_ = 0;
u8 destroyedStructures = 0;
u8 turtlesKilled = 0;

u8 checkObjectives()
{
    if (remainingTurns == 0)
    {
        // Player ran out of turns so the game is over.
        failObjectives();
    }

    // Fail historical structures map if too many structures are destroyed (scaled with difficulty).
    if (destroyedStructures > allowedDestroyedStructures)
    {
        failObjectives();
    }

    // Fail wildlife level if too many turtles are killed (scaled with difficulty).
    if (turtlesKilled > allowedKilledTurtles)
    {
        failObjectives();
    }

    // If all the mines have been defused\destroyed then the level is complete.
    // This can apply to all levels even though its not necessarily possible to achieve on soem levels.
    if (mineCount == 0)
    {
        return TRUE;
    }

    // If the content of the platforms has been delivered to the destination then the level is complete.
    if (currentLevel == LEVEL_RESCUE)
    {
        if (rescueCargo_ == 0)
        {
            return TRUE;
        }
    }

    // If the ships have all been rescued then the level is complete.
    if (currentLevel == LEVEL_GATED_TRADELANE ||
        currentLevel == LEVEL_TRADELANE ||
        currentLevel == LEVEL_SACRIFICE ||
        currentLevel == LEVEL_ESCORT)
    {
        if (shipsToRescue_ == 0)
        {
            return TRUE;
        }

        // If non player ships get destroyed then they lose the level.
        if (currentLeveShipArraySize / 5 - friendlyShipCount > allowedDestroyedShips)
        {
            failObjectives();
        }
    }

    // Only on the sacrifice level are you allowed to lose all your units.
    if (currentLevel != LEVEL_SACRIFICE)
    {
        u8 i = 0;
        u8 deadUnits = 0;

        for (i = 0; i < playerUnitCount; ++i)
        {
            deadUnits += units[i].dead;
        }

        if (deadUnits == playerUnitCount)
        {
            // All player units are dead so lose the level.
            failObjectives();
        }
    }

    // Objectives have not been completed.
    return FALSE;
}

u8 failObjectives()
{
    // Level is uncompletable so the level has ended.
    levelEnded = LEVEL_END_DEATH;
}

void initObjectives()
{
    // Saving cargo.
    if (currentLevel == LEVEL_RESCUE)
    {
        rescueCargo_ = 6;
    }

    // Saving ships.
    if (currentLevel == LEVEL_GATED_TRADELANE)
    {
        shipsToRescue_ = 2;
    }
    else if (currentLevel == LEVEL_TRADELANE)
    {
        shipsToRescue_ = 3;
    }
    else if (currentLevel == LEVEL_SACRIFICE)
    {
        shipsToRescue_ = 4;
    }
    else if (currentLevel == LEVEL_ESCORT)
    {
        shipsToRescue_ = 1;
    }

    destroyedStructures = 0;
    turtlesKilled = 0;
}

void updateObjectives()
{
    if (currentLevel == LEVEL_RESCUE)
    {
        --rescueCargo_;
    }

    if (currentLevel == LEVEL_GATED_TRADELANE || 
        currentLevel == LEVEL_TRADELANE || 
        currentLevel == LEVEL_SACRIFICE || 
        currentLevel == LEVEL_ESCORT)
    {
        --shipsToRescue_;
    }
}