#ifndef UNIT_H
#define UNIT_H

#include <cpctelera.h>
#include "GameState.h"
#include "GameTypes.h"
#include "Interactable.h"
#include "Interrupt.h"
#include "Keys.h"
#include "Map.h"
#include "Movement.h"
#include "Object.h"
#include "Objectives.h"
#include "Scrolling.h"
#include "SoundManager.h"
#include "sprites/UnitSprites.h"
#include "sprites/ShipSprites.h"

#define PLAYER_UNIT_CLASS 1
#define SHIP_UNIT_CLASS 2
#define MARINE_LIFE_UNIT_CLASS 3

#define UNIT_TYPE_DIVER 0
#define UNIT_TYPE_EXPLOSIVE_DIVER 1
#define UNIT_TYPE_KAMAKAZI 2
#define UNIT_TYPE_MAGNET 3

#define UNIT_TYPE_RADAR 4
#define UNIT_TYPE_TRANSPORT 5
#define UNIT_TYPE_CABLED 6
#define UNIT_TYPE_CABLED_PAIR 7

// The unit graphic indexes for transport.
#define UNIT_GRAPHIC_INDEX_EMPTY_TRANSPORT 11
#define UNIT_GRAPHIC_INDEX_FULL_TRANSPORT 29

// This matches the index of the ship in the ship graphics array.
#define UNIT_TYPE_TURTLE 15

// This matches the index of the ship in the ship graphics array.
#define UNIT_TYPE_WARSHIP 16

#define MARINE_LIFE_MONEY_DEDUCTION 300

// Amount of money gained from salvaging a mine with a magnet minesweeper.
#define MAGNET_MINE_SALVAGE_MONEY_ADD 150

typedef struct Unit Unit;

struct Unit
{
    Position position;
    u8 type;
    u8 deployed;
    u8 width;
    u8 height;
    u8 direction;
    Position nextPosition;
    u8 health;
    u8 unitClass;
    Position target;
    u8 defusing;
    u8 dead;
    u8 graphicIndex;
    u8 id;
    u8 targetStartIndex;
    u8 targetEndIndex;
    u8 carryingCargo;
};

// All the units in the game.
extern Unit units[MAX_UNIT_COUNT];

// The number of units that the player currently has deployed to the map.
extern u8 playerUnitCount;

// Flag indicating if the player unit is ready for their turn or not.
extern u8 playerUnitTurnFlag;

// Indicates if a unit or friendly ship can be found at the position.
u8 containsUnit(u8 x, u8 y, Unit* unit);

// Detonates a big bang when the kamakazi ship hits a mine.
void deliverKamikaziPayload(Unit* unit);

// Draws the units in the game on screen.
void drawUnits();

// Initialises units in the game.
void initUnits();

// Executes the logic for the active units in the game.
void logicUnits();

// Executes the logic for moving a unit (player or non-player) to its next position or target.
void logicUnitToNextPosition(Unit* unit, u8 unitIndex);

// Moves the ship position based on x,y and next x,y.
void moveUnits();

// Detects if a cell that a unit is on is empty or contains something that needs to be processed.
u8 processCellUnit(Unit* unit, u8 x, u8 y);

// Determines if the next position (x,y) for the unit is solid or not by checking unit dimensions.
u8 unitNextPositionIsSolid(Unit* unit, u8 nextPosition);

// Updates the mine type around the radar ship following a radar scan (radar defuse).
u8 updateMinesAroundUnit(u8 x, u8 y, u8 unitWidth, u8 unitHeight, u8 radius, u8 mineTypeToFind, u8 mineTypeToSet);

#endif