#include "UnitSelection.h"

#define UNIT_SELECTION_TITLE_ "choose units"
#define TITLE_X 28
#define TITLE_Y 1

#define MONEY_Y 16
#define MONEY_TEXT "money"
#define MONEY_VALUE_X 42

#define SLOTS_AVAILABLE_TEXT "slots"
#define SLOTS_AVAILABLE_Y 31
#define SLOTS_AVAILABLE_X 32
#define SLOTS_VALUE_X 42

#define UNIT_X 8
#define UNIT_Y_0 32
#define UNIT_Y_1 64
#define UNIT_Y_2 96
#define UNIT_Y_3 128
#define UNIT_Y_3_PART2 144
#define UNIT_RIGHT_X 60

#define UNIT_MONEY_LEFT_X 21
#define UNIT_MONEY_RIGHT_X 53

#define UNIT_COUNT_X 1
#define UNIT_COUNT_RIGHT_X 77

#define UNIT_BOX_LEFT_X 0
#define UNIT_BOX_RIGHT_X 76

#define UNIT_COUNT_0_Y 35
#define UNIT_COUNT_1_Y 67
#define UNIT_COUNT_2_Y 99
#define UNIT_COUNT_3_Y 131

// The maximum number of units of any given type that the player can have normally.
#define MAX_PLAYER_UNIT_OF_TYPE_COUNT 4

u8 selectedUnitCount_ = 0;
u8 selectedUnit_ = 0;
static u8 screenText_[6];
static u8 unitCounts_[MAX_PLAYER_UNIT_COUNT];

void drawUnitBalances()
{
    u8* memory = 0;
    u8 i = 0;

    // Remaining unit count that the player can chose.
    drawString(TITLE_X, SLOTS_AVAILABLE_Y, SLOTS_AVAILABLE_TEXT);
    drawCharacter(SLOTS_VALUE_X, SLOTS_AVAILABLE_Y, MAX_PLAYER_UNIT_COUNT - selectedUnitCount_);

    // Player's current balance.
    convertNumberToString(gameMoney, screenText_);
    drawString(TITLE_X, MONEY_Y, MONEY_TEXT);
    drawString(MONEY_VALUE_X, MONEY_Y, screenText_);

    // Draw boxes over where the cursor and unit counts go.
    memory = cpct_getScreenPtr(CPCT_VMEM_START, UNIT_BOX_LEFT_X, UNIT_Y_0);
    cpct_drawSolidBox(memory, cpctm_px2byteM0(0, 0), 4, 112);
    cpct_drawSolidBox(memory + UNIT_BOX_RIGHT_X, cpctm_px2byteM0(0, 0), 4, 112);

    // Draws the cursor over the selected unit.
    if (selectedUnit_ < 4)
    {
        // 0 - 3 is on the left column.
        // +1 to do 32x1, 32x2 etc.
        memory = cpct_getScreenPtr(CPCT_VMEM_START, UNIT_BOX_LEFT_X, UNIT_Y_0 * (selectedUnit_ + 1));
        cpct_drawSpriteMasked(G_cursor_white, memory, TILE_WIDTH_BYTES, TILE_HEIGHT);
    }
    else
    {
        // 4 - 7 is on the right column.
        // -3 to do 32x1, 32x2 etc.
        memory = cpct_getScreenPtr(CPCT_VMEM_START, UNIT_BOX_RIGHT_X, UNIT_Y_0 * (selectedUnit_ - 3));
        cpct_drawSpriteMasked(G_cursor_white, memory, TILE_WIDTH_BYTES, TILE_HEIGHT);
    }

    // Drawing the number, money of each type of unit selected.
    for (i = 0; i < 4; ++i)
    {
        // Number of units.
        drawCharacter(UNIT_COUNT_X, 3 + UNIT_Y_0 * (i + 1), unitCounts_[i]);
        drawCharacter(UNIT_COUNT_RIGHT_X, 3 + UNIT_Y_0 * (i + 1), unitCounts_[i + 4]);

        // Cost of units.
        convertNumberToString(unit_money[i], screenText_);
        drawString(UNIT_MONEY_LEFT_X, 4 + UNIT_Y_0 * (i + 1), screenText_);

        convertNumberToString(unit_money[i + 4], screenText_);
        drawString(UNIT_MONEY_RIGHT_X, 4 + UNIT_Y_0 * (i + 1), screenText_);
    }

    drawString(15, 170, "press deploy key once done");
}

void runUnitSelection()
{
    u8* memory = cpct_getScreenPtr(CPCT_VMEM_START, UNIT_X, UNIT_Y_0);
    u8 i = 0;
    static u8 maxUnitType[MAX_PLAYER_UNIT_COUNT];

    // Point to the unitGraphics array.
    u8** unitGraphic = unitGraphics;

    // Initialise unit counts to 0.
    for (i = 0; i < MAX_PLAYER_UNIT_COUNT; ++i)
    {
        unitCounts_[i] = 0;

        if (currentLevel == LEVEL_SACRIFICE || currentLevel == LEVEL_ESCORT || currentLevel == LEVEL_RESCUE)
        {
            maxUnitType[i] = 0;
        }
        else
        {
            maxUnitType[i] = MAX_PLAYER_UNIT_OF_TYPE_COUNT;
        }
    }

    if (currentLevel == LEVEL_SACRIFICE)
    {
        // Only kamakazi units on sacrifice level.
        maxUnitType[UNIT_TYPE_KAMAKAZI] = MAX_PLAYER_UNIT_COUNT;
    }
    else if (currentLevel == LEVEL_ESCORT)
    {
        // Only radar and transport for escort level.
        maxUnitType[UNIT_TYPE_RADAR] = MAX_PLAYER_UNIT_OF_TYPE_COUNT;
        maxUnitType[UNIT_TYPE_TRANSPORT] = 1;
    }
    else if (currentLevel == LEVEL_RESCUE)
    {
        // Only radar and transport for rescue level.
        maxUnitType[UNIT_TYPE_RADAR] = MAX_PLAYER_UNIT_OF_TYPE_COUNT;
        maxUnitType[UNIT_TYPE_TRANSPORT] = MAX_PLAYER_UNIT_OF_TYPE_COUNT;
    }

    selectedUnitCount_ = 0;
    selectedUnit_ = 0;

    // Clear the screen so we can display the unit selection options.
    cpct_clearScreen(0);

    // Title of the screen in approximately the centre of the screen.
    drawString(TITLE_X, TITLE_Y, UNIT_SELECTION_TITLE_);

    memory = cpct_getScreenPtr(CPCT_VMEM_START, UNIT_X, UNIT_Y_0);

    // Left column of units.
    for (i = 0; i < 8; ++i)
    {
        cpct_drawSprite(*unitGraphic, memory, TILE_WIDTH_BYTES, TILE_HEIGHT);

        // The first two units are 1x1 so memory offset is different.
        if (i < 2)
        {
            memory += 320;
        }
        else
        {
            if (i % 3 == 1)
            {
                // Move to the next unit line.
                memory += 312;
            }
            else
            {
                // Move to the next part of the unit on the same line.
                memory += 4;
            }
        }

        // Move to the next graphic in the unit graphic array.
        unitGraphic++;
    }

    // Right column of units.
    memory = cpct_getScreenPtr(CPCT_VMEM_START, UNIT_RIGHT_X, UNIT_Y_0);
    
    for (i = 0; i < 12; ++i)
    {
        cpct_drawSprite(*unitGraphic, memory, TILE_WIDTH_BYTES, TILE_HEIGHT);

        if (i % 3 == 2)
        {
            // Move to the next unit line.
            memory += 312;
        }
        else
        {
            // Move to the next part of the unit on the same line.
            memory += 4;
        }

        // Move to the next graphic in the unit graphic array.
        unitGraphic++;
    }

    // Move memory directly under the minesweeper pair graphic.
    memory -= 160;

    for (i = 0; i < 3; ++i)
    {
        cpct_drawSprite(*unitGraphic, memory, TILE_WIDTH_BYTES, TILE_HEIGHT);

        // Move to the next part of the unit on the same line.
        memory += 4;

        // Move to the next graphic in the unit graphic array.
        unitGraphic++;
    }

    // Draw the unit and slot balances on the screen.
    drawUnitBalances();

    // Reading controls to choose player units.
    while (1)
    {
        readGameControls();

        if (gameControls.controlFlags & CONTROL_DOWN_FLAG)
        {
            // Move the cursor down to select a lower unit.
            if (selectedUnit_ + 1 < MAX_PLAYER_UNIT_COUNT)
            {
                ++selectedUnit_;
            }
        }
        else if (gameControls.controlFlags & CONTROL_UP_FLAG)
        {
            // Move the cursor up to select an upper unit.
            if (selectedUnit_ > 0)
            {
                --selectedUnit_;
            }
        }
        else if (gameControls.controlFlags & CONTROL_RIGHT_FLAG)
        {
            // There are only 8 slots in the game so the player can only have that number of units.
            if (selectedUnitCount_ < MAX_PLAYER_UNIT_COUNT)
            {
                // Make sure the player does not have more than the allowed number of units of any given type.
                if (unitCounts_[selectedUnit_] < maxUnitType[selectedUnit_])
                {
                    // Make sure the player has enough money.
                    if (gameMoney >= unit_money[selectedUnit_])
                    {
                        ++unitCounts_[selectedUnit_];
                        ++selectedUnitCount_;

                        // Spend game money when unit is added.
                        gameMoney -= unit_money[selectedUnit_];
                    }
                }
            }
        }
        else if (gameControls.controlFlags & CONTROL_LEFT_FLAG)
        {
            if (unitCounts_[selectedUnit_] > 0)
            {
                --unitCounts_[selectedUnit_];
                --selectedUnitCount_;

                // Restore game money when unit is removed.
                gameMoney += unit_money[selectedUnit_];
            }
        }

        if (gameControls.controlFlags)
        {
            // Draw the unit and slot balances on the screen.
            drawUnitBalances();

            // Wait for any previous key presses to clear.
            while (cpct_isAnyKeyPressed());

        #ifdef WINCPCTELERA
            // Prevents moving up and down too fast.
            while (!cpct_isAnyKeyPressed());
        #endif
        }

        // Exiting the unit screen.
        if (selectedUnitCount_ && (gameControls.controlFlags & CONTROL_USE_FLAG))
        {
            break;
        }

        // Check that the exit key has been pressed.
        if (gameControls.controlFlags & CONTROL_EXIT_FLAG)
        {
            // If the player choses to press yes exit the screen.
            if (exitDialog())
            {
                levelEnded = LEVEL_END_DEATH;
                return;
            }
            else
            {
                runUnitSelection();
            }
        }
    }

    // Adds units to the unit array based on chosen unit counts.
    selectedUnit_ = 0;

    for (i = 0; i < MAX_PLAYER_UNIT_COUNT; ++i)
    {
        u8 j = 0;

        // Adding all units in the count to the units array.
        for (j = 0; j < unitCounts_[i]; ++j)
        {
            units[selectedUnit_].type = i;
            units[selectedUnit_].graphicIndex = unit_graphic_index[i];
            units[selectedUnit_++].deployed = FALSE;
        }
    }

    // Update the player unit count based on the selection.
    playerUnitCount = selectedUnit_;
}